const Order = require("../Order");
const dayjs = require("dayjs");

class OrderReceipt {
  /**
   *
   * @param {Order} order
   */
  static CREATE_ORDER_RECEIPT(order, invoice) {
    let letter_left_margin = OrderReceipt.CM_TO_POINTS(2.5, false);
    let letter_right_margin = OrderReceipt.CM_TO_POINTS(2, false);

    let PDFDocument = require("pdfkit");
    let title = `Bestellung ${order.getOrderID()}`;
    if (invoice) {
      title = `Rechnung INV_${order.getOrderID()}`;
    }
    const doc = new PDFDocument({
      size: "A4",
      info: {
        Title: title,
        Author: "SUMA-EV - Verein für freien Wissenszugang",
        Subject: "MetaGer Schlüssel: Suchanfragen (x300)",
      },
    });

    doc
      .font("public/fonts/liberation-sans/LiberationSans-Regular.ttf")
      .fontSize(10);

    // The header with logo and line
    doc.image("public/images/metager.png", letter_left_margin, 30, {
      height: 20,
    });
    doc.image("public/images/suma-ev.png", 400, 32, {
      height: 18,
    });
    doc
      .moveTo(letter_left_margin, 60)
      .lineTo(OrderReceipt.CM_TO_POINTS(21, false) - letter_right_margin, 60)
      .strokeColor("#515151")
      .stroke();

    // Our Address Information
    if (invoice) {
      doc
        .fontSize(8)
        .text(
          "SUMA-EV | Röselerstraße 3 | 30159 Hannover | Deutschland",
          OrderReceipt.CM_TO_POINTS(2, false),
          OrderReceipt.CM_TO_POINTS(4.5, true),
          {
            width: OrderReceipt.CM_TO_POINTS(8.5, false),
          }
        )
        .moveDown()
        .fontSize(10)
        .text(invoice.name);
      invoice.address.split("\r\n").forEach((line) => {
        doc.text(line);
      });
    }

    // General Information
    let text = invoice ? "Rechnung:" : "Bestellnummer:";
    let number = invoice ? "INV_" + order.getOrderID() : order.getOrderID();
    doc
      .fontSize(12)
      .text(
        text,
        OrderReceipt.CM_TO_POINTS(21 - 8.5, false),
        OrderReceipt.CM_TO_POINTS(5, true),
        {
          width: OrderReceipt.CM_TO_POINTS(7.5, false),
          continued: true,
          lineGap: 4,
        }
      )
      .font("public/fonts/liberation-sans/LiberationSans-Bold.ttf")
      .text(number, { align: "right" })
      .font("public/fonts/liberation-sans/LiberationSans-Regular.ttf")
      .text("Telefon:", {
        width: OrderReceipt.CM_TO_POINTS(7.5, false),
        lineGap: 4,
        continued: true,
      })
      .text("+4951134000070", { align: "right" })
      .text("E-Mail:", {
        width: OrderReceipt.CM_TO_POINTS(7.5, false),
        lineGap: 4,
        continued: true,
      })
      .text("support@metager.de", { align: "right" })
      .text("Internet:", {
        width: OrderReceipt.CM_TO_POINTS(7.5, false),
        lineGap: 4,
        continued: true,
      })
      .text("metager.de", { align: "right" })
      .moveDown()
      .text("Datum:", {
        width: OrderReceipt.CM_TO_POINTS(7.5, false),
        lineGap: 4,
        continued: true,
      })
      .text(dayjs().format("DD.MM.YYYY"), { align: "right" });

    // Actual content of the letter
    // Orange Line
    doc
      .moveTo(
        OrderReceipt.CM_TO_POINTS(2.5, false),
        OrderReceipt.CM_TO_POINTS(12.6, true)
      )
      .lineTo(
        OrderReceipt.CM_TO_POINTS(21 - 2, false),
        OrderReceipt.CM_TO_POINTS(12.6, true)
      )
      .strokeColor("#ff7f00")
      .stroke();

    text = invoice ? "Rechnung" : "Auftragsbestätigung";
    doc
      .fontSize(12)
      .font("public/fonts/liberation-sans/LiberationSans-Bold.ttf")
      .text(
        text + " " + number,
        OrderReceipt.CM_TO_POINTS(2.5, false),
        OrderReceipt.CM_TO_POINTS(9.846, true)
      );

    doc
      .font("public/fonts/liberation-sans/LiberationSans-Bold.ttf")
      .text(
        "Bestelldetails",
        OrderReceipt.CM_TO_POINTS(2.5, false),
        OrderReceipt.CM_TO_POINTS(12, true),
        {
          width: OrderReceipt.CM_TO_POINTS(10, false),
          align: "center",
        }
      )
      .font("public/fonts/liberation-sans/LiberationSans-Regular.ttf")
      .moveDown()
      .text("MetaGer Schlüssel: Suchanfragen (300x)", {
        width: OrderReceipt.CM_TO_POINTS(10, false),
        align: "right",
      });

    doc
      .font("public/fonts/liberation-sans/LiberationSans-Bold.ttf")
      .text(
        "Anzahl",
        OrderReceipt.CM_TO_POINTS(12.5, false),
        OrderReceipt.CM_TO_POINTS(12, true),
        {
          width: OrderReceipt.CM_TO_POINTS(3.25, false),
          align: "center",
        }
      )
      .font("public/fonts/liberation-sans/LiberationSans-Regular.ttf")
      .moveDown()
      .text(order.getAmount() / 300, {
        width: OrderReceipt.CM_TO_POINTS(3.25, false),
        align: "right",
        lineGap: 8,
      })
      .text("MwSt (7%)", {
        width: OrderReceipt.CM_TO_POINTS(3.25, false),
        align: "right",
        lineGap: 8,
      })
      .font("public/fonts/liberation-sans/LiberationSans-Bold.ttf")
      .text("Gesamtbetrag", {
        width: OrderReceipt.CM_TO_POINTS(3.25, false),
        align: "right",
      });

    doc
      .font("public/fonts/liberation-sans/LiberationSans-Bold.ttf")
      .text(
        "Preis",
        OrderReceipt.CM_TO_POINTS(15.75, false),
        OrderReceipt.CM_TO_POINTS(12, true),
        {
          width: OrderReceipt.CM_TO_POINTS(3.25, false),
          align: "center",
        }
      )
      .font("public/fonts/liberation-sans/LiberationSans-Regular.ttf")
      .moveDown()
      .text(order.getNettoPrice().toFixed(2) + " €", {
        width: OrderReceipt.CM_TO_POINTS(3.25, false),
        align: "right",
        lineGap: 8,
      })
      .text(order.getVatAmount().toFixed(2) + " €", {
        width: OrderReceipt.CM_TO_POINTS(3.25, false),
        align: "right",
        lineGap: 8,
      })
      .font("public/fonts/liberation-sans/LiberationSans-Bold.ttf")
      .text(order.getPrice().toFixed(2) + " €", {
        width: OrderReceipt.CM_TO_POINTS(3.25, false),
        align: "right",
      })
      .font("public/fonts/liberation-sans/LiberationSans-Regular.ttf");
    if (invoice) {
      doc
        .font("public/fonts/liberation-sans/LiberationSans-Bold.ttf")
        .text(
          "Rechnungsbetrag dankend erhalten!",
          OrderReceipt.CM_TO_POINTS(2.5, false),
          OrderReceipt.CM_TO_POINTS(18, true),
          {
            align: "center",
          }
        )
        .font("public/fonts/liberation-sans/LiberationSans-Regular.ttf");
    }

    // Footer
    doc
      .moveTo(
        OrderReceipt.CM_TO_POINTS(2.5, false),
        OrderReceipt.CM_TO_POINTS(27.2, false)
      )
      .lineTo(
        OrderReceipt.CM_TO_POINTS(21 - 2, false),
        OrderReceipt.CM_TO_POINTS(27.2, true)
      )
      .strokeColor("#515151")
      .stroke();

    let textbox_width = doc.widthOfString("Vorstand");
    doc.fontSize(8);
    textbox_width = Math.max(
      doc.widthOfString("Dominik Hebeler", "Manuela Branz", "Carsten Riel")
    );
    doc.fontSize(10);

    let x = OrderReceipt.CM_TO_POINTS(21 - 2, false) - textbox_width;
    doc
      .font("public/fonts/liberation-sans/LiberationSans-Bold.ttf")
      .text("Vorstand", x, OrderReceipt.CM_TO_POINTS(27.3, true), {
        lineBreak: false,
      })
      .fontSize(8)
      .moveDown(1.5)
      .font("public/fonts/liberation-sans/LiberationSans-Regular.ttf");
    doc.x = x;
    doc
      .text("Dominik Hebeler", {
        lineBreak: false,
      })
      .moveDown();
    doc.x = x;
    doc
      .text("Manuela Branz", {
        lineBreak: false,
      })
      .moveDown();
    doc.x = x;
    doc.text("Carsten Riel", {
      lineBreak: false,
    });

    x = OrderReceipt.CM_TO_POINTS(2.5, false);
    doc
      .fontSize(10)
      .font("public/fonts/liberation-sans/LiberationSans-Bold.ttf")
      .text("SUMA-EV", x, OrderReceipt.CM_TO_POINTS(27.3, true), {
        lineBreak: false,
      })
      .fontSize(8)
      .moveDown(1.5)
      .font("public/fonts/liberation-sans/LiberationSans-Regular.ttf");
    doc.x = x;
    doc
      .text("Röselerstraße 3", {
        lineBreak: false,
      })
      .moveDown();
    doc.x = x;
    doc
      .text("30159 Hannover", {
        lineBreak: false,
      })
      .moveDown();
    doc.x = x;
    doc.text("Deutschland", {
      lineBreak: false,
    });

    doc.end();

    const chunks = [];
    return new Promise((resolve, reject) => {
      doc.addListener("data", (chunk) => chunks.push(chunk));
      doc.addListener("error", (err) => reject(err));
      doc.addListener("end", () => resolve(chunks));
    });
  }

  /**
   *
   * @param {Number} cm
   * @param {boolean} height
   * @returns
   */
  static CM_TO_POINTS(cm, vertical) {
    let width_in_points = 595.28;
    let height_in_points = 841.89;
    if (vertical) {
      return cm * (height_in_points / 29.7);
    } else {
      return cm * (width_in_points / 21.0);
    }
  }
}

module.exports = OrderReceipt;
