var express = require("express");
var router = express.Router({ mergeParams: true });

const { param, validationResult } = require("express-validator");
const Order = require("../../app/Order");
const OrderReceipt = require("../../app/pdf/OrderReceipt");

router.use("/", (req, res, next) => {
  if (typeof req.cookies.order !== "undefined") {
    req.data.form = {
      "order-id": req.cookies.order,
    };
  }
  next();
});
router.get("/", (req, res, next) => {
  req.data.page = "order";
  req.data.css.push(`${res.locals.baseDir}/styles/orders/orders.css`);
  res.render("key", req.data);
});

router.post("/", (req, res, next) => {
  let matches = req.body["order-id"].match(/^(INV_)?(\d+)$/);
  if (!matches) {
    req.data.page = "order";
    req.data.css.push(`${res.locals.baseDir}/styles/orders/orders.css`);
    req.data.error = 400;
    req.data.form = {
      "order-id": req.body["order-id"],
    };
    res.render("key", req.data);
  } else {
    let order_id = matches[2];
    Order.LOAD_ORDER_FROM_ID(order_id)
      .then(/** @param {Order} order */(order) => order.getKeyFromOrderLink())
      .then((key) => {
        if (key !== req.data.key.key.get_key()) {
          throw "Order is not connected to this key";
        }
        res.redirect(`${res.locals.baseDir}/key/${req.data.key.key.get_key()}/orders/${order_id}`);
      })
      .catch((reason) => {
        req.data.page = "order";
        req.data.css.push(`${res.locals.baseDir}/styles/orders/orders.css`);
        req.data.error = 404;
        req.data.form = {
          "order-id": req.body["order-id"],
        };
        res.render("key", req.data);
      });
  }
});

router.use("/:order_id", param("order_id").isInt(), (req, res, next) => {
  const errors = validationResult(req);
  if (!errors.isEmpty()) {
    return res.status(404).render("error", {
      message: "Order not found",
      error: {
        status: 404,
      },
    });
  }
  Order.LOAD_ORDER_FROM_ID(req.params.order_id)
    .then((order) => {
      res.cookie("order", order.getOrderID());
      req.data.page = "order";
      req.data.css.push(`${res.locals.baseDir}/styles/orders/order.css`);
      req.data.order = { order: order };
      req.data.links.order_url = `${res.locals.baseDir}/key/${req.data.key.key.get_key()}/orders/${req.params.order_id}#order`;
      req.data.links.receipt_url = `${res.locals.baseDir}/key/${req.data.key.key.get_key()}/orders/${req.params.order_id}/pdf`;
      req.data.links.invoice_url = `${res.locals.baseDir}/key/${req.data.key.key.get_key()}/orders/${req.params.order_id}/invoice#invoice-form`;
      req.data.links.refund_url = `${res.locals.baseDir}/key/${req.data.key.key.get_key()}/orders/${req.params.order_id}/refund#refund-form`;
      if (req.data.order.order.isReceiptCreated()) {
        req.data.order.download_invoice_url = `${res.locals.baseDir}/key/${req.data.key.key.get_key()
          }/orders/${req.data.order.order.getOrderID()}/invoice/download`;
      }
      next("route");
    })
    .catch((reason) => {
      res.status(404).render("error", {
        message: "Order not found",
        error: {
          status: 404,
        },
      });
    });
});

router.get("/:order_id", (req, res) => {
  res.render("key", req.data);
});

router.get("/:order_id/pdf", (req, res) => {
  OrderReceipt.CREATE_ORDER_RECEIPT(req.data.order.order)
    .then((data) => {
      res
        .status(200)
        .header({
          "Content-Type": "application/pdf",
        })
        .send(Buffer.concat(data));
    })
    .catch((reason) => {
      res.locals.error = {
        status: 500,
        stack: JSON.stringify(reason, null, 4),
      };
      res.locals.message = "Error generating Order Receipt";
      res.status(500).render("error");
    });
});

let invoiceRouter = require("./invoice");
router.use("/:order_id/invoice", invoiceRouter);

let refundRouter = require("./refund");
router.use("/:order_id/refund", refundRouter);

module.exports = router;
